/* Expire old NNTP messages. */

#include "global.h"
#if defined(EXPIRY) && defined(NNTPS)
#ifdef MSDOS
#include <dir.h>
#include <dos.h>
#endif
#include <sys/stat.h>
#include "ctype.h"
#include <time.h>
#include "timer.h"
#include "proc.h"
#include "bm.h"
#include "dirutil.h"
#include "hardware.h"
#include <limits.h>


#define DAYStoSECS (24L*60L*60L)
int Eproc = 0;

static char OpenErr[] = "expire: err %d opening %s";
static char RenErr[] = "expire: err %d renaming %s to %s";
static char WriteErr[] = "expire: err %d writing %s";


/* NNTP Expire stuff */
void update_nntp_history (int);
void expire_nntpsrv (char *, int);
void update_nntp_active (void);
static int convert_num (char **);
static void update_newsrc (int, int, char *);
static char *newsgroup_to_path (char *);
void RemoveNntpLocks (void);



/* expire NNTP articles
   coded by brett england (future amateur) and rob vk5xxx.
*/
static int
convert_num (char **p)
{
int i;
char *pp = *p;

	i = (*pp - '0') * 10 + *(pp + 1) - '0';
	(*p) += 2;
	return (i);
}



void
update_nntp_history (int age)
{
FILE *old, *new;
char newfile[FILE_PATH_SIZE];
char line[LINELEN];
char *p;
struct tm tm_time;
time_t expire_time, file_time, now;
int i, history_records_expired = 0;

#ifdef notdef
	for (i = 10; i > 0; i--) {
		if (!mlock (History, NULLCHAR))
			break;
		mspause (2000L);
	}
	if (!i)
		return;		/* can't lock, we'll have to wait until next time */
#endif

	expire_time = (long) age *DAYStoSECS;	/* days to seconds */

	(void) time (&now);

	sprintf (newfile, "%s.new", History);
	unlink (newfile);
	if ((old = fopen (History, READ_TEXT)) == NULLFILE) {
		log (-1, OpenErr, errno, History);
#ifdef notdef
		rmlock (History, NULLCHAR);
#endif
		return;
	}
	if ((new = fopen (newfile, WRITE_TEXT)) == NULLFILE) {
		fclose (old);
		log (-1, OpenErr, errno, newfile);
#ifdef notdef
		rmlock (History, NULLCHAR);
#endif
		return;
	}
	/* Scan the history file, deleting old article references */
	/* format: <message-id> yymmdd hhmmss ngname/Art# */
	for ( ; ; ) {
		pwait (NULL);
		if (fgets (line, LINELEN, old) == NULL)
			break;

		p = line;
		while (*p != '\0' && *p != '>')
			p++;
		if (*p != '>')	/* Some sort of corrupt line */
			continue;
		p += 2;
		tm_time.tm_year = convert_num (&p);
		if (tm_time.tm_year < 70)
			tm_time.tm_year += 100;	/* 21-st century */
		tm_time.tm_mon = convert_num (&p) - 1;
		tm_time.tm_mday = convert_num (&p);
		p++;		/* Point to time component */
		tm_time.tm_hour = convert_num (&p);
		tm_time.tm_min = convert_num (&p);
		tm_time.tm_sec = 0;

		file_time = mktime (&tm_time);

		if (file_time > (now - expire_time))
			fputs (line, new);
		else
			history_records_expired++;
	}
	i = ferror (new);
	fclose (new);
	fclose (old);

	if (!i) {
		unlink (History);
		if (rename (newfile, History) == -1)
			log (-1, RenErr, errno, newfile, History);
	} else
		log (-1, WriteErr, errno, newfile);

#ifdef notdef
	rmlock (History, NULLCHAR);
#endif

	if (history_records_expired > 0)
		log (-1, "Expired: %d in %s", history_records_expired, History);
}



/* Check the news.rc file if the number contained is less than minval
   replace the news.rc file contents with minval.
*/
static void
update_newsrc (int minval, int maxval, char *path)
{
FILE *newsrcfd;
char news_file[FILE_PATH_SIZE], line[LINELEN];
int no;

	sprintf (news_file, "%s/news.rc", path);
	if ((newsrcfd = fopen (news_file, "r+")) == NULL) {
		/* not that serious!  log(-1,OpenErr, errno, news_file);*/
		return;
	}
	(void) fgets (line, LINELEN, newsrcfd);
	rewind (newsrcfd);

	no = atoi (line);
	if ((no < minval) || (no > maxval))
		fprintf (newsrcfd, "%d", minval);
	fclose (newsrcfd);

}



/* Update the active file and return the lowest news article
*/
void
update_nntp_active (void)
{
FILE *old, *new;
char newfile[FILE_PATH_SIZE];
char line[LINELEN];
char *p, *rpath;
const char *path;
int minval, maxval;
int command, file_num;
struct ffblk *file;

	sprintf (newfile, "%s.new", Active);
	unlink (newfile);
	if ((old = fopen (Active, READ_TEXT)) == NULLFILE) {
		log (-1, OpenErr, errno, Active);
		return;
	}
	if ((new = fopen (newfile, WRITE_TEXT)) == NULLFILE) {
		fclose (old);
		log (-1, OpenErr, newfile);
		return;
	}
	file = (struct ffblk *) mallocw (sizeof (struct ffblk));

	/* rewrite the active file, with current maxval and minval article numbers */
	/* format: ngname MaxArt# MinArt# Postflag */
	for (;;) {
		pwait (NULL);
		if (fgets (line, LINELEN, old) == NULL)
			break;

		p = line;
		while (*p && !isspace (*p))
			p++;
		if (*p == '\0')
			continue;
		*p = '\0';

		if ((rpath = newsgroup_to_path (line)) == NULLCHAR) {
			*p = ' ';	/* weird error ... copy line unchanged to be safe */
			fprintf (new, "%s", line);
			continue;
		}
		p++;
		while (*p && (isspace (*p) || isdigit (*p)))
			p++;

		path = wildcardize ((const char *) rpath);

		command = 0;
		minval = INT_MAX;
		maxval = INT_MIN;
		for (;;) {
			pwait (NULL);
			if (!nextname (command, path, file))
				break;
			command = 1;	/* Find next */

			if (file->ff_name[0] == '.')
				continue;	/* ignore . and .. */

			file_num = atoi (file->ff_name);
			if (file_num > 0) {	/* should always be greater than 0 */
				if (file_num < minval)
					minval = file_num;
				if (file_num > maxval)
					maxval = file_num;
			}
		}
		if (minval == INT_MAX && maxval == INT_MIN) {	/* No files */
			minval = 1;
			maxval = 0;
		}
		update_newsrc (minval, maxval, rpath);

		fprintf (new, "%s %05d %05d %s", line, maxval, minval, p);
	}

	command = ferror (new);
	fclose (new);
	fclose (old);
	free (file);

	if (!command) {
		unlink (Active);
		if (rename (newfile, Active) == -1)
			log (-1, RenErr, newfile, Active);
	} else
		log (-1, WriteErr, errno, newfile);
}



static char *
newsgroup_to_path (char *group)
{
FILE *f;
static char line[LINELEN];
char *cp, *cp1;

	if ((f = fopen (Pointer, "r")) == NULLFILE)
		return (NULLCHAR);

	for (;;) {
		pwait (NULL);
		if (fgets (line, LINELEN, f) == NULLCHAR)
			break;

		if (strcspn (line, " ") != strlen (group))
			continue;

		if (strnicmp (group, line, strlen (group)) == 0) {
			cp = strchr (line, ' ');
			if (!cp)
				continue;
			cp++;
			if ((cp1 = strchr (cp, ' ')) != NULLCHAR)
				*cp1 = '\0';	/* drop date-created */
			else
				rip (cp);
			fclose (f);
			return (cp);
		}
	}
	fclose (f);
	return (NULLCHAR);
}



void
expire_nntpsrv (char *nntp_name, int age)
{
const char *path;
int command = 0;
struct ffblk file;
struct tm tm_time;
time_t file_time, expire_time, now;
char nntp_file[FILE_PATH_SIZE];
char save_path[FILE_PATH_SIZE];
int file_num, expired = 0;

	/* Resolve nntp name into directory path */
	if ((path = newsgroup_to_path (nntp_name)) == NULLCHAR)
		return;

	expire_time = (long) age *DAYStoSECS;

	(void) time (&now);
	strcpy (save_path, path);
	path = wildcardize ((const char *) path);

	for (;;) {
		pwait (NULL);
		if (!nextname (command, path, &file))
			break;
		command = 1;	/* Find next */

		if (file.ff_name[0] == '.')
			continue;	/* ignore . and .. */

		file_num = atoi (file.ff_name);
		if (file_num > 0) {
			/* only expire a file that is numeric :-) */
			tm_time.tm_sec = 0;	/* DOS doesn't store this */
#if defined(TNOS_68K) || defined(UNIX)
			tm_time.tm_min = file.ff_ftime.tm_min;
			tm_time.tm_hour = file.ff_ftime.tm_hour;
			tm_time.tm_mday = file.ff_ftime.tm_mday;
			tm_time.tm_mon = file.ff_ftime.tm_mon;
			tm_time.tm_year = file.ff_ftime.tm_year;
#else
			tm_time.tm_min = (file.ff_ftime >> 5) & 0x3f;
			tm_time.tm_hour = (file.ff_ftime >> 11) & 0x1f;
			tm_time.tm_mday = file.ff_fdate & 0x1f;
			tm_time.tm_mon = ((file.ff_fdate >> 5) & 0xf) - 1;
			tm_time.tm_year = (file.ff_fdate >> 9) + 80;
#endif

			file_time = mktime (&tm_time);

			if ((now - expire_time) > file_time) {
				sprintf (nntp_file, "%s/%s", save_path, file.ff_name);
				unlink (nntp_file);
				expired++;
			}
		}
	}
	if (expired)
		log (-1, "Expired: %d in %s", expired, nntp_name);
}



void
RemoveNntpLocks (void)
{				/* called by main() during startup phase */
FILE *actf;
char line[LINELEN];
char *p, *rpath;

	rmlock (History, NULLCHAR);
	/* remove locks for any newsgroup named in the active file */

	if ((actf = fopen (Active, READ_TEXT)) == NULLFILE)
		return;

	for (;;) {
		if (fgets (line, LINELEN, actf) == NULLCHAR)
			break;

		p = line;
		while (*p && !isspace (*p))
			p++;
		if (*p == '\0')
			continue;	/* malformed */
		*p = '\0';

		/* newsgroup_to_path() exists? */
		if ((rpath = newsgroup_to_path (line)) != NULLCHAR)
			rmlock (rpath, NULLCHAR);
	}
	fclose (actf);
}

#endif /* EXPIRY */
